#!/usr/bin/env python
# -*- coding: UTF8 -*-

### Copyright (C) 2006-2009 Antonio Valentino <a_valentino@users.sf.net>

### This file is part of BESTGUI.

### BESTGUI is free software; you can redistribute it and/or modify
### it under the terms of the GNU General Public License as published by
### the Free Software Foundation; either version 2 of the License, or
### (at your option) any later version.

### BESTGUI is distributed in the hope that it will be useful,
### but WITHOUT ANY WARRANTY; without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
### GNU General Public License for more details.

### You should have received a copy of the GNU General Public License
### along with BESTGUI; if not, write to the Free Software
### Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.

'''Main application module.'''

__author__  = 'Antonio Valentino <a_valentino@users.sf.net>'
__date__    = '$Date: 2009-04-13 10:21:31 +0200 (lun, 13 apr 2009) $'
__version__ = '$Revision: 705 $'

import re
import os
import sys
import time
import glob
import locale
import shutil
import logging
import warnings
import traceback
import webbrowser
import cStringIO
import ConfigParser

from operator import attrgetter

import gtk
import pango
import gobject

if gtk.pygtk_version< (2,6,0):
    print 'PyGtk 2.6.0 or later required'
    raise SystemExit

import info
import menu
import utils
import xopen
import errors
import dialogs
import viewers
import paramset
import gladewrap
import preferences
import filepatterns
import exectoolsgtk

from exectools import EX_OK

CUSTOM_TOOL_HMI = {
    'header.ini': paramset.HeaderAnalysisParamSet,
    'quick.ini': paramset.QuickLookParamSet,
    'full.ini': paramset.FullResolutionParamSet,
    #'impras.ini': paramset.ImportRasterParamSet, @TODO: add
    'coreg.ini': paramset.CoRegistrationParamSet,
    # @TODO: hide Sensor Id
    #~ 'adc.ini': paramset.ADCParamSet,
    #~ 'envibacksca.ini': paramset.EnviBackscaParamSet,
    #~ 'ersbacksca.ini': paramset.ErsBackscaParamSet,
}

class BestOutputHandler(exectoolsgtk.GtkOutputHandler):
    def __init__(self, textview, statusbar, progressbar=None, blinker=None,
                 tags=None):
        super(BestOutputHandler, self).__init__(textview, statusbar,
                                                progressbar, blinker)
        self.context_id = statusbar.get_context_id('running')
        self.texttags = {
            'error': re.compile('error|warning|bad input|parameter not '
                                'defined|not found|invalid parameter',
                                re.IGNORECASE),
            'warning': re.compile('warning', re.IGNORECASE),
            'completed': re.compile('completed', re.IGNORECASE),
        }

        if tags is None:
            tags = {
                'error': dict(foreground='red'),
                'warning': dict(foreground='orange'),
                'info': dict(foreground='blue'),
                'debug': dict(foreground='gray'),
                'cmd': dict(weight=pango.WEIGHT_BOLD),
                'completed': dict(foreground='blue'),
                #'mesage': dict(),
            }

        if tags:
            textbuffer = self.textview.get_buffer()
            tagtable = textbuffer.get_tag_table()
            for tag_name, props in tags.items():
                if tagtable.lookup(tag_name) or not props:
                    continue
                textbuffer.create_tag(tag_name, **props)

    def handle_line(self, data):
        if 'Doing' in data:
            self.statusbar.pop(self.context_id)
            self.statusbar.push(self.context_id, data.strip())
        #~ elif ('Insert volume' in data) or
             #~ ('Press <Enter> to continue ...' in data):
            #~ # @TODO: pop up a dialog
            #~ pass

        # skip empty lines
        if data.isspace():
            return

        # this also flushes
        super(BestOutputHandler, self).handle_line(data)

class ParamView(object):
    title_format = '<span size="xx-large"><b>%s</b></span>'

    def __init__(self, paramform, textview=None):
        super(ParamView, self).__init__()
        if textview is None:
            # use gtksourceview if available
            try:
                import gtksourceview2
                logging.info('using gtksourceview2')

                manager = gtksourceview2.language_manager_get_default()
                lang = manager.get_language('ini')

                srcbuffer = gtksourceview2.Buffer()
                srcbuffer.set_language(lang)
                #srcbuffer.set_highlight_syntax(True)   # should be the default
                #srcbuffer.set_check_brackets(True)     # should be the default

                textview = gtksourceview2.View(srcbuffer)
                textview.set_show_line_numbers(True)
                textview.set_show_line_marks(True)
                textview.set_show_right_margin(True)
                textview.set_highlight_current_line(True)

                # @TODO: actions to add/remove marks
            except ImportError:
                try:
                    import gtksourceview
                    logging.info('using gtksourceview (version 1)')

                    manager = gtksourceview.SourceLanguagesManager()
                    languages = manager.get_available_languages()
                    lang = [l for l in languages if '.ini' in l.get_name()][0]

                    srcbuffer = gtksourceview.SourceBuffer()
                    srcbuffer.set_language(lang)
                    srcbuffer.set_highlight(True)
                    #srcbuffer.set_check_brackets(True) # should be the default

                    textview = gtksourceview.SourceView(srcbuffer)
                    textview.set_show_line_numbers(True)
                    textview.set_show_line_markers(True)
                    textview.set_show_margin(True)
                except ImportError:
                    logging.debug('gtksourceview not available')
                    textview = gtk.TextView()

        self.paramform = paramform
        self.textview = textview

        self.scrolledwin = gtk.ScrolledWindow()
        self.scrolledwin.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.scrolledwin.add(textview)
        self.widget = gtk.Frame()
        self.widget.set_shadow_type(gtk.SHADOW_NONE)
        self.widget.set_label_align(0.5, 0.5)
        self.widget.add(self.paramform.widget)
        self.widget.show_all()
        self._mode = 'form' # or 'text'

    def get_label(self):
        return self.paramform.label

    label = property(get_label)

    def _sync_text(self):
        cfg = ConfigParser.SafeConfigParser()
        try:
            self.paramform.update_cfg(cfg)
        except errors.MissingParameterError:
            pass
        data = cStringIO.StringIO()
        cfg.write(data)

        textbuffer = self.textview.get_buffer()
        textbuffer.set_text(data.getvalue())

    def _sync_form(self):
        textbuffer = self.textview.get_buffer()
        data = textbuffer.get_text(textbuffer.get_start_iter(),
                                   textbuffer.get_end_iter())
        fd = cStringIO.StringIO(data)
        cfg = ConfigParser.SafeConfigParser()
        cfg.readfp(fd)
        self.paramform.set_from_cfg(cfg, errors_='warn')

    def get_mode(self):
        return self._mode

    def set_mode(self, mode):
        if mode == self._mode:
            return

        if mode == 'text':
            self.widget.remove(self.paramform.widget)
            self.widget.add(self.scrolledwin)
            self.scrolledwin.show_all()
            label = gtk.Label()
            label.set_markup(self.title_format % self.label)
            label.show()
            self.widget.set_label_widget(label)
            self._sync_text()
        elif mode == 'form':
            self.widget.set_label(None)
            self.widget.remove(self.scrolledwin)
            self.widget.add(self.paramform.widget)
            self.paramform.widget.show_all()
            self._sync_form()
        else:
            raise ValueError('invalid mode: "%s"' % mode)
        self._mode = mode

    mode = property(get_mode, set_mode, doc='View mode ("form" or "text").')

    def load(self, filename):
        self.paramform.load(filename) # performs all checks
        if self.mode == 'text':
            try:
                fd = open(filename)
            except TypeError:
                fd = filename
            try:
                data = fd.read()
                textbuffer = self.textview.get_buffer()
                textbuffer.set_text(data)
            finally:
                fd.close()

    def save(self, filename):
        if self.mode == 'form':
            return self.paramform.save(filename)
        else:
            textbuffer = self.textview.get_buffer()
            data = textbuffer.get_text(textbuffer.get_start_iter(),
                                       textbuffer.get_end_iter())
            try:
                fd = open(filename, 'w')
                try:
                    fd.write(data)
                    return True
                finally:
                    fd.close()
            except (OSError, IOError):
                pass
            return False

    def set_default(self):
        self.paramform.set_default()
        if self.mode == 'text':
            self._sync_text()

    def clear(self):
        self.paramform.clear()
        if self.mode == 'text':
            self._sync_text()

    def get_enabled(self):
        return self.widget.get_property('sensitive')

    def set_enabled(self, enabled):
        self.widget.set_sensitive(enabled)

    enabled = property(get_enabled, set_enabled)

_force_timeout_mode = False
#~ _force_timeout_mode = True

if (sys.platform[:3] == 'win') or _force_timeout_mode:
    Executor = exectoolsgtk.GtkTimeoutExecutor
    #Executor.timeout = 50 # ms
else:
    Executor = exectoolsgtk.GtkIOWatchExecutor

class BestGUI(gladewrap.GtkGladeApp, Executor):
    PARAMFILE_PATTERNS = (('*.ini', 'Parameters files'), ('*', 'All Files'))
    LOGFORMAT = '%(asctime)-8s %(filename)-15s %(funcName)-18s %(lineno)-5s %(message)s'
    LOGDATEFMT = '%x %X'

    def __init__(self, loglevel=None):
        # Logo setup

        # The gtk+-2.10.13-setup.exe package from gimp-win.sf.net doesn't
        # provide the svg loader.
        # @TODO: fix
        if sys.platform[:3] == 'win':
            logo = utils.get_iconfile('bestguilogo128.png')
        else:
            logo = utils.get_iconfile('bestguilogo.svg')

        gtk.window_set_default_icon_from_file(logo)

        # Base class initialization
        gladefile = utils.get_gladefile('bestgui.glade')
        gladewrap.GtkGladeApp.__init__(self, gladefile, 'mainwin',
                                       info.version)

        # GUI setup
        self.mainwin.set_title('BESTGUI v. %s' % info.version)
        self.background_image.set_from_file(logo)
        self.notebook.set_show_tabs(False)

        menumap, hmimap, patternsmap = self.scan_config()
        self.build_tools_menu(menumap)
        filepatterns.set_patterns_map(patternsmap)

        context_id = self.statusbar.get_context_id('ready')
        self.statusbar.push(context_id, ' Ready')
        self.vpaned.set_position(self.vpaned.get_property('max-position'))

        self.setup_hooks()

        # Set the attributes
        self._sys_excepthook = None
        self._status = None
        self.paramview = None
        self.resultview = None
        self.inifilename = None
        self.logger = None
        self.hmimap = hmimap

        self.setup_logging(loglevel)

        ### Settings ###
        # Get the configuration file name
        # @TODO: improve
        cfgfilename = utils.get_user_cfgfile()
        if cfgfilename and not os.path.exists(cfgfilename):
            default_cfgfilename = utils.get_cfgfile('bestgui.ini')
            shutil.copyfile(default_cfgfilename, cfgfilename)
        elif not cfgfilename:
            cfgfilename = utils.get_cfgfile('bestgui.ini')

        self.settings = preferences.Settings()
        width  = self.mainwin.get_screen().get_width() * 3/4
        height = self.mainwin.get_screen().get_height() * 3/4
        self.settings.window_size = (width, height)
        try:
            self.settings.load(cfgfilename)
        except errors.BestEnvError, e:
            dialogs.run_error_dialog(message=e)
            self.edit_options()

        # @TODO: remove this method; use subbrocess.Popen env parameter instaed
        if loglevel is None:
            self.logger.setLevel(self.settings.loglevel)

        if self.settings.window_size:
            self.mainwin.resize(*self.settings.window_size)
        if self.settings.window_position:
            self.mainwin.move(*self.settings.window_position)

        # Output plane and execuroe setup
        self.outputplane = exectoolsgtk.GtkOutputPlane(
                                self.outputview,
                                hide_callback=self.on_hide_outputplane)

        ### Executor initialization ###
        outputhandler = BestOutputHandler(self.outputview, self.statusbar,
                                          tags=self.settings.tags)

        Executor.__init__(self, outputhandler, logger=self.logger)

        # Final initializations
        self.set_status('ready')
        logging.debug('Initialization done. log level = %s.' % logging.getLevelName(logging.getLogger().level))

    ### Properties ############################################################
    def _get_workdir(self):
        # @TODO: track last opened
        return os.path.expanduser(self.settings.workdir)

    workdir = property(_get_workdir)

    ### Setup helpers #########################################################
    def scan_config(self):
        # @TODO: use caching
        menumap = menu.get_menu_map()
        configdir = utils._configdir # @TODO: fix
        hmimap = {}
        patternsmap = {}
        for hmifilename in glob.glob(os.path.join(configdir, 'hmi', '*.ini')):
            cfg = ConfigParser.SafeConfigParser()
            cfg.read(hmifilename)

            title    = cfg.get('MAIN', 'title')
            label    = cfg.get('MAIN', 'patterns_label')
            patterns = cfg.get('MAIN', 'patterns')
            if patterns != 'ALL':
                patterns = tuple([pattern.strip()
                                    for pattern in patterns.split(',')])

            name     = cfg.get('MENU', 'name')
            parent   = cfg.get('MENU', 'parent')
            label    = cfg.get('MENU', 'label')
            position = cfg.get('MENU', 'position')
            position = int(position)

            data  = os.path.basename(hmifilename)

            item = menu.MenuItemDescriptor(name, parent, label,
                                           position, data)

            menumap[parent].children.append(item)
            hmimap[title] = hmifilename
            patternsmap[title] = filepatterns.FilePattern(label, patterns)

        for menu_ in menumap.values():
            menu_.children.sort(key=attrgetter('position'))

        return menumap, hmimap, patternsmap

    def build_tools_menu(self, menumap):
        toolsmenu = menu.build_tools_menu(menumap, self.on_tool_selected)
        self.toolsmenu.set_submenu(toolsmenu)

    def setup_logging(self, level=logging.INFO):
        # WARNING: this method should never be called more than once

        self.logger = logging.getLogger()
        formatter = logging.Formatter('%(levelname)s: %(message)s')
        handler = exectoolsgtk.GtkStreamLoggingHandler(self.outputview)
        #handler.setLevel(level) # @TODO: check
        handler.setFormatter(formatter)
        self.logger.addHandler(handler)

        formatter = logging.Formatter('%(message)s')
        handler = exectoolsgtk.GtkDialogLoggingHandler(parent=self.mainwin)
        handler.setLevel(logging.WARNING)
        handler.setFormatter(formatter)
        handler.dialog.set_title('BESTGUI')
        self.logger.addHandler(handler)

        if level is not None:
            self.logger.setLevel(level)

    def setup_hooks(self):
        '''Set the about dialog hooks'''

        def email_hook(about_dialog, email, user_data):
            # @TODO: look at gazpacho-0.7.2/gazpacho/app/bugreportdialog.py
            subject = '[BESTGUI %s]' % info.version
            return xopen.mailto(email, subject=subject)

        def url_hook(about_dialog, url, app):
            return app.open_url(url)

        gtk.about_dialog_set_email_hook(email_hook, None)
        gtk.about_dialog_set_url_hook(url_hook, self)

    def build_tool_GUI(self, hmifilename):
        try:
            # Setup the paramsForm
            # @TODO: improve plugin selection system
            try:
                customtool = CUSTOM_TOOL_HMI[os.path.basename(hmifilename)]
                paramform = customtool(workdir=self._get_workdir)
            except KeyError:
                paramform = paramset.MainCfgParamSet(hmifilename,
                                                     workdir=self._get_workdir)
            paramform.clear()
            paramform.widget.show_all()
            paramview = ParamView(paramform)

            # Setup the main frame
            pagenum = self.notebook.page_num(self.background_image)
            self.notebook.remove_page(pagenum)
            self.notebook.append_page(paramview.widget)
            title = paramview.label + ' parameters'
            self.notebook.set_tab_label_text(paramview.widget, title.title())
            self.notebook.set_show_tabs(False)

            # Set instance attributes
            self.paramview = paramview
        except IOError:
            dialogs.run_error_dialog()

    ### Execution control methods #############################################
    def main(self):
        '''Set the custom exception handler and run the GTK main loop.'''

        self._sys_excepthook = sys.excepthook
        sys.excepthook = self.excepthook
        gtk.main()

    def quit(self, widget=None):
        '''Stop the GTK main loop and reset the default except hook.'''

        try:
            if self.subprocess:
                self.stop()
        except: # @TODO: check
            pass

        try:
            if self.resultview:
                self.resultview.close() # remove the temporary folder
        except: # @TODO: check
            pass

        try:
            if self.mainwin.get_property('visible'):
                self.mainwin.unmaximize()
                self.save_window_state()
            self.settings.save()
        except:
            self.logger.exception('unable to save preferences')

        gtk.main_quit()
        sys.excepthook = self._sys_excepthook

    def excepthook(self, etype, evalue, etraceback):
        '''Customized excepthook

        Executes the following operations:
            1) stop the running sub-process (best) if any
            2) show the exception infos in a (modal) dialog window
            3) destroy application
            4) call original excepthook

        '''

        try:
            try:
                if self.subprocess:
                    self.subprocess.stop()
                banner = ('# BESTGUI v. %s\n# Unhandled ecception report '
                          'generated on %s\n\n') % (info.version,
                                                    time.asctime())
                dialogs.run_unhandledexception_dialog(self.mainwin, banner,
                                                    etype, evalue, etraceback)
                self.quit()
            except: # @TODO: check
                sys.stderr.write('Exception in "excepthook".\n')
                traceback.print_exc(file=sys.stderr)
        finally:
            self._sys_excepthook(etype, evalue, etraceback)
            sys.exit(1)

    ### Internal and utility methods ##########################################
    def edit_options(self, cfg=None):
        if not cfg:
            cfg = self.settings.get_cfg()
        dialog = preferences.OptionsDialog(self.settings.filename, cfg)

        done = False
        while not done:
            try:
                response = dialog.run()
                #~ dialog.hide()
                cfg = dialog.get_options()
                self.settings.update(cfg)
            except errors.BestEnvError, e:
                if response == gtk.RESPONSE_CANCEL:
                    done = True
                    dtype = gtk.MESSAGE_WARNING
                else:
                    dtype = gtk.MESSAGE_ERROR
                dialogs.run_message_dialog(e, self.mainwin, dtype)
            else:
                done = True
                if response == gtk.RESPONSE_OK:
                    self.settings.save()
        dialog.destroy()

        self.logger.setLevel(self.settings.loglevel)

    def save_window_state(self):
        if not self.mainwin.get_property('visible'):
            return

        self.flush_events()
        self.settings.window_size = self.mainwin.get_size()
        self.settings.window_position = self.mainwin.get_position()

        # @TODO: check
        #~ if self.outputplane_isopen():
            #~ self.settings.outputplane_fraction = self.outputplane_fraction
        #~ else:
            #~ self.settings.outputplane_fraction = 0.

    def open_url(self, url):
        try:
            return self.settings.browserlauncher.open(url)
        except (webbrowser.Error, OSError), e:
            # WindowsError is a subclass of OSError
            dialogs.run_error_dialog(e)

    def set_status(self, status):
        # If the new status equals the current status no action is performed
        # except for the 'open' value. In this case the action is performed
        # anyway order to handle situations in which a new file is opened or
        # the current file is saved with a new name.
        if status == self._status and status != 'open':
            return

        if status == 'ready':
            if self._status not in (None, 'new', 'open'):
                warnings.warn('%s --> ready' % self._status)
                context_id = self.statusbar.get_context_id(self._status)
                self.statusbar.pop(context_id)

            # File actions
            getattr(self, 'open_menuitem').set_sensitive(True)
            getattr(self, 'open_button').set_sensitive(True)

            for name in ('save', 'saveas', 'show_text', 'reload', 'close'):
                getattr(self, '%s_menuitem' % name).set_sensitive(False)
                getattr(self, '%s_button' % name).set_sensitive(False)
            # Run actions
            for name in ('run', 'stop', 'clear', 'default'):
                getattr(self, '%s_menuitem' % name).set_sensitive(False)
                getattr(self, '%s_button' % name).set_sensitive(False)
            # Tools Menu
            for item in self.toolsmenu.get_submenu().get_children():
                item.set_sensitive(True)

            # Outout plane
            self.show_outputplane(False)

            if self.show_text_button.get_active():
                self._status = status
                self.show_text_menuitem.set_active(False)
                self.show_text_button.set_active(False)

            # Status bar
            context_id = self.statusbar.get_context_id('open')
            self.statusbar.pop(context_id)

        elif status == 'new':
            self.paramview.enabled = True
            if self._status not in ('ready', 'open', 'textview', 'running'):
                warnings.warn('%s --> new' % self._status)
                context_id = self.statusbar.get_context_id(self._status)
                self.statusbar.pop(context_id)

            # File actions
            for name in ('open', 'save', 'saveas', 'reload', 'close'):
                getattr(self, '%s_menuitem' % name).set_sensitive(True)
                getattr(self, '%s_button' % name).set_sensitive(True)

            # Run actions
            self.stop_button.set_sensitive(False)
            self.stop_menuitem.set_sensitive(False)
            for name in ('run', 'clear', 'default'):
                getattr(self, '%s_menuitem' % name).set_sensitive(True)
                getattr(self, '%s_button' % name).set_sensitive(True)

            # Preferences actions
            self.show_text_menuitem.set_sensitive(True)
            self.show_text_button.set_sensitive(True)

            # Tools Menu
            for item in self.toolsmenu.get_submenu().get_children():
                item.set_sensitive(False)

            # Status bar
            context_id = self.statusbar.get_context_id('open')
            self.statusbar.pop(context_id)
            self.statusbar.push(context_id, ' *Untitled parameters file')

        elif status == 'open':
            if self._status not in ('ready', 'new', 'open', 'running'):
                warnings.warn('%s --> open' % self._status)
                context_id = self.statusbar.get_context_id(self._status)
                self.statusbar.pop(context_id)

            if self._status != 'new':
                self.set_status('new')

            # Status bar
            inifilename = self.inifilename # @TODO: basename (??)
            context_id = self.statusbar.get_context_id('open')
            self.statusbar.pop(context_id)
            self.statusbar.push(context_id, ' File name: %s' % inifilename)

        elif status == 'running':
            if self._status not in ('open',):
                warnings.warn('%s --> running' % self._status)
                context_id = self.statusbar.get_context_id(self._status)
                self.statusbar.pop(context_id)

            # Param form
            self.paramview.enabled = False

            # File actions
            for name in ('open', 'save', 'saveas', 'reload', 'close'):
                getattr(self, '%s_menuitem' % name).set_sensitive(False)
                getattr(self, '%s_button' % name).set_sensitive(False)

            # Run actions
            self.stop_button.set_sensitive(True)
            self.stop_menuitem.set_sensitive(True)
            for name in ('run', 'clear', 'default'):
                getattr(self, '%s_menuitem' % name).set_sensitive(False)
                getattr(self, '%s_button' % name).set_sensitive(False)

            # Preferences actions
            self.show_text_menuitem.set_sensitive(False)
            self.show_text_button.set_sensitive(False)

            # output plane
            self.show_outputplane()

            # Status Bar
            context_id = self.statusbar.get_context_id('running')
            self.statusbar.push(context_id, 'Running...')

        else:
            raise ValueError('Unknown status value: "%s"' % str(status))

        self._status = status

    def open(self, inifilename, parent=None):
        # @TODO: fix (I don't like too much the parent parameter here)
        cfg = ConfigParser.SafeConfigParser()
        try:
            cfg.read(inifilename)
        except ConfigParser.Error, e:
            logging.error('invalid file format.\n\n%s' % e)
            return
        except (OSErro, IOError), e:
            logging.error('unable to read parameters file', exc_info=True)
            return

        sections = cfg.sections()

        if self.paramview and self.paramview.label in sections:
            self.close_resultview()
            try:
                self.paramview.load(inifilename)
            except ValueError, e:
                dialogs.run_error_dialog(str(e))
            else:
                self.inifilename = inifilename
        else:
            if self.paramview:
                msg = ('"%s" doesn\'t seem to be a valid parameter '
                       'file for %s.\nOpen it anyway?') % (
                             os.path.basename(inifilename),
                             self.paramview.label)
                dialog = gtk.MessageDialog(
                            parent  = parent,
                            type    = gtk.MESSAGE_QUESTION,
                            flags   = gtk.DIALOG_MODAL |
                                      gtk.DIALOG_DESTROY_WITH_PARENT,
                            buttons = gtk.BUTTONS_YES_NO,
                            message_format = msg)
                response = dialog.run()
                dialog.destroy()
                if response == gtk.RESPONSE_YES:
                    self.on_close()
                else:
                    return

            # @TODO: fix for multi-section files
            if len(sections) == 0:
                msg = 'Invalid parameters file "%s".' % inifilename
                dialogs.run_message_dialog(msg, parent)
                self.logger.info(msg) # @TODO: check
                return
            toolname = sections[0]
            try:
                hmifilename = self.hmimap[toolname]
            except KeyError, e:
                raise errors.BestGUIException(
                    'Unable to locate the HMI file for "%s".' % toolname)
            hmifilename = utils.get_hmifile(hmifilename)
            self.build_tool_GUI(hmifilename)
            try:
                self.paramview.load(inifilename)
            except ValueError, e:
                dialogs.run_error_dialog(str(e))
                self.on_close()
                return
            self.inifilename = inifilename
        self.set_status('open')

    def close_resultview(self):
        if self.resultview:
            pagenum = self.notebook.page_num(self.resultview.widget)
            if pagenum != -1:
                self.notebook.remove_page(pagenum)
                if self.notebook.get_n_pages() < 2:
                    self.notebook.set_show_tabs(False)
            self.resultview.close()
            self.resultview = None

    def reset(self):
        super(BestGUI, self).reset()
        self.set_status('open')

    def run(self, filename):
        args = (self.settings.bestbin, '-if', filename)
        env = os.environ.copy()
        env.update(self.settings.best_env)
        super(BestGUI, self).run(args, self.workdir, env)

    def prerun_hook(self, cmd):
        if not isinstance(cmd, basestring):
            cmd = map(os.path.basename, cmd)
            cmd = ' '.join(cmd)

        prompt = '$' # or '#'
        self.outputplane.write('%s %s\n' % (prompt, cmd), 'cmd')
        logging.debug('%s %s\n' % (prompt, cmd))

        self.set_status('running')
        self.outputhandler.progressbar.show()
        #super(BestGUI, self).prerun_hook(args)

    def finalize_hook(self):
        if (self.subprocess.returncode != EX_OK or self.error_flag
                                                or self._stopped):
            self.close_resultview()
        else:
            if self.resultview is not None:
                return

            tools_without_preview = (
                # extraction
                'MEDIA ANALYSIS',
                'COORDINATES RETRIEVING',
                'SUPPORT DATA',
                'INGESTION XCA',
                # conversion
                'GEOMETRIC CONVERSION',        # @TODO: provide a custom viewer
                'SENSITIVITY EVALUATION',
                # stats
                'GLOBAL STATISTIC',            # @TODO: provide a custom viewer
                'PRINCIPAL COMPONENT ANALYSIS',# @TODO: provide a custom viewer
                # coregister
                'IMAGE CO-REGISTRATION',
                'DORIS BASELINE EVALUATION',
                # @TODO: complete
            )
            tools_with_custom_preview = (
                # extraction
                'HEADER ANALYSIS',
                'QUICK LOOK',
                # conversion
                'ANCILLARY DATA DUMP',
                #'GEOMETRIC CONVERSION',         # @TODO
                # stats
                #'GLOBAL STATISTIC',             # @TODO
                #'PRINCIPAL COMPONENT ANALYSIS', # @TODO
            )

            toolname = self.paramview.paramform.label

            if toolname in tools_without_preview:
                return

            filename = self.get_outimage_filename()
            if filename:
                ext = os.path.splitext(filename)[-1]

                # @WARNING: using a private module variable
                if ext not in filepatterns._all_patterns and \
                                    toolname not in tools_with_custom_preview:
                    return
            elif toolname not in tools_with_custom_preview:
                return

            # Only ask is there actually is a preview to show and the flag
            # is 'ask'
            if self.settings.show_preview.lower() == 'ask':
                dialog = gtk.MessageDialog(
                            self.mainwin,
                            gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
                            gtk.MESSAGE_QUESTION,
                            gtk.BUTTONS_YES_NO,
                            'Do you want to generate a preview of the '
                            'processing outputs?')
                dialog.set_default_response(gtk.RESPONSE_YES)
                hbox = gtk.HBox()
                radiobutton = None
                for label in ('Always', 'Ask', 'Never'):
                    radiobutton = gtk.RadioButton(radiobutton, label)
                    hbox.pack_end(radiobutton, expand=False, padding=3)
                hbox.show_all()
                dialog.vbox.pack_end(hbox)
                group = radiobutton.get_group()
                group[1].set_active(True)

                response = dialog.run()
                for radiobutton in group:
                    if radiobutton.get_active():
                        show_preview = radiobutton.get_label().lower()
                        self.settings.show_preview = show_preview
                        break
                dialog.destroy()
                if response == gtk.RESPONSE_NO:
                    return
            elif self.settings.show_preview.lower() == 'never':
                return

            # @TODO: complete
            if toolname == 'HEADER ANALYSIS':
                self.resultview = viewers.ESAAnnotationsViewer()
                self.resultview.annotations_file = \
                        self.get_outimage_filename('Annotation File','.txt')

            elif toolname == 'QUICK LOOK':
                self.resultview = viewers.QuickLookViewer()
                self.resultview.ql_imagefile = self.get_outimage_filename(
                                                'Output Quick Look Image')
                self.resultview.qlg_imagefile = self.get_outimage_filename(
                                                    'Output Grid Image')
            elif toolname == 'ANCILLARY DATA DUMP':
                # use fiename already computed
                self.resultview = viewers.AnnotationsViewer()
                self.resultview.dumpfile = \
                        self.get_outimage_filename('Output File')

            #~ elif toolname == 'GEOMETRIC CONVERSION':
                #~ pass

            #~ elif toolname == 'GLOBAL STATISTIC':
                #~ pass

            #~ elif toolname == 'PRINCIPAL COMPONENT ANALYSIS':
                #~ pass

            else:
                # use fiename already computed
                if os.path.splitext(filename)[-1] == '.tif':
                    self.resultview = viewers.ImageViewer()
                    self.resultview.imagefile = filename
                else:
                    # BEST product in internal format
                    self.resultview = viewers.BESTProductViewer(filename,
                                                                self.run)

    def finalize(self):
        super(BestGUI, self).finalize()
        if self.resultview:
            if (hasattr(self.resultview, 'generate') and
                                                self.resultview.needs_update):
                msg = 'generating image preview ...'
                context_id = self.statusbar.get_context_id('preview')
                self.statusbar.push(context_id, msg)
                logging.info(msg)
                self.resultview.generate(self.settings.workdir)
            else:
                context_id = self.statusbar.get_context_id('preview')
                self.statusbar.pop(context_id)
                if self.notebook.page_num(self.resultview.widget) == -1:
                    # the result view tab does not exist
                    self.notebook.append_page(self.resultview.widget)
                    self.notebook.set_tab_label_text(self.resultview.widget,
                                                     'Results View')
                    self.notebook.next_page()
                    self.notebook.set_show_tabs(True)
                try:
                    self.resultview.show_contents()
                except (IOError, gobject.GError):
                    logging.debug('unable to show preview', exc_info=True)
                    self.close_resultview()

                # @TODO: all 'show_contents' implementations should return
                #        True on success
                #~ if not self.resultview.show_contents():
                    #~ self.close_resultview()

    def get_outimage_filename(self, outfile_param='Output Image', ext=None):
        cfg = ConfigParser.SafeConfigParser()
        self.paramview.paramform.update_cfg(cfg)
        section = cfg.sections()[-1] # there shoud be only one section
        if cfg.has_option(section, outfile_param):
            try:
                dirname = cfg.get(section, 'Output Dir')
                filename = cfg.get(section, outfile_param)
            except ConfigParser.NoOptionError, e:
                warnings.warn(('unable to find "%s" in section "%s". '
                               'Image preview skipped.') % (e.option,
                                                            e.section))
                return

            dirname = utils.remove_quote(dirname)
            if not os.path.isabs(dirname):
                dirname = os.path.join(self.workdir, dirname)
                dirname = os.path.normpath(dirname)
            filename = utils.remove_quote(filename)
            filename = os.path.join(dirname, filename)

            if ext is None:
                if section == 'FULL RESOLUTION':
                    # @TODO: set extension
                    ext = None
                else:
                    # get the file extension
                    if cfg.has_option(section, 'Input Image'):
                        infilename = cfg.get(section, 'Input Image')
                    else:
                        infilename = None
                    ext = filepatterns.get_ext(section, infilename)
            if ext:
                filename = filename + ext
            else:
                matches = glob.glob(filename + '.???')
                exclude = ('.ini', '.HAN', '.txt')
                matches = [match for match in matches
                                if os.path.splitext(match)[-1] not in exclude]
                if len(matches) == 1:
                    filename = matches[0]
                else:
                    logging.warning('unable to find the output file '
                                    'name for preview generation')
                    return

            return filename

    def outputplane_isopen(self):
        pos = self.vpaned.get_position()
        maxpos = self.vpaned.get_property('max-position')
        return pos < maxpos

    def show_outputplane(self, flag=True):
        if flag == self.outputplane_isopen():
            return
        if flag:
            self.show_output_menuitem.set_active(True)
            self.show_output_button.set_active(True)
        else:
            self.show_output_menuitem.set_active(False)
            self.show_output_button.set_active(False)

    ### Callbacks for the File menu ###########################################
    def on_open(self, widget=None):
        filedialog = gtk.FileChooserDialog(
                            title='Open BEST parameters file',
                            parent=self.mainwin,
                            buttons=(gtk.STOCK_OK,     gtk.RESPONSE_OK,
                                     gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
        dialogs.add_filechooser_filters(filedialog, self.PARAMFILE_PATTERNS)
        # @TODO: store last opened
        filedialog.set_current_folder(self.workdir)
        response = filedialog.run()
        if response == gtk.RESPONSE_OK:
            inifilename = filedialog.get_filename()
            self.open(inifilename)
        filedialog.destroy()

    def on_save(self, widget=None):
        if not self.paramview:
            warnings.warn('"on_save" callback invoked with no tool opened.')
            return False
        inifilename = self.inifilename
        if inifilename:
            saved = self.paramview.save(inifilename)
            if saved:
                self.set_status('open')
            return saved
        else:
            return self.on_saveas(widget)#, workdir) # @TODO: check

    def on_saveas(self, widget=None):
        filedialog = gtk.FileChooserDialog(
                            title   = 'Save As',
                            parent  = self.mainwin,
                            action  = gtk.FILE_CHOOSER_ACTION_SAVE,
                            buttons = (gtk.STOCK_OK,     gtk.RESPONSE_OK,
                                       gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))

        filedialog.set_select_multiple(False)
        filedialog.set_default_response(gtk.RESPONSE_OK)

        dialogs.add_filechooser_filters(filedialog, self.PARAMFILE_PATTERNS)
        # @TODO: chech it is actually an absolute path
        inifilename = self.inifilename
        if inifilename:
            filedialog.set_filename(inifilename)
        else:
            filedialog.set_current_name('parameters.ini')
            if self.settings.workdir:
                filedialog.set_current_folder(self.settings.workdir)

        inifilename = dialogs.run_save_dialog(filedialog)
        filedialog.destroy()
        if inifilename:
            self.inifilename = inifilename
            self.set_status('open')
            return self.paramview.save(inifilename)

    def on_show_text_toggled(self, widget=None):
        if self.paramview is None:
            return

        state = widget.get_active()
        #self.paramview.mode = widget.get_active() and 'text' or 'form'
        if state:
            self.paramview.mode = 'text'
        else:
            self.paramview.mode = 'form'

        # sync buttons state
        if self.show_text_button.get_active() != state:
            self.show_text_button.set_active(state)
        if self.show_text_menuitem.get_active() != state:
            self.show_text_menuitem.set_active(state)

    def on_reload(self, widget=None):
        inifilename = self.inifilename
        self.paramview.load(inifilename)

    def on_close(self, widget=None):
        # check changes on params win
        self.show_outputplane(False)
        self.close_resultview()

        pagenum = self.notebook.page_num(self.paramview.widget)
        self.notebook.remove_page(pagenum)
        #self.paramview.close() # @TODO: ask for user confirmation (??)
        self.paramview = None

        self.notebook.append_page(self.background_image)

        if self.notebook.get_n_pages() < 2:
            self.notebook.set_show_tabs(False)

        context_id = self.statusbar.get_context_id('open')
        self.statusbar.pop(context_id)
        self.set_status('ready')

    ### Callbacks for the Edit menu ###########################################

    ### Callbacks for the Tools menu ##########################################
    def on_tool_selected(self, widget=None, data=None):
        #~ if data is None:
            #~ data = @TODO: complete
        hmifilename = utils.get_hmifile(data)
        self.build_tool_GUI(hmifilename)
        self.inifilename = None
        self.set_status('new')

    ### Callbacks for the Run menu ############################################
    def on_run(self, widget=None):
        if not self.on_save():
            # @NOTE: this also appens when the user clicks on the cancel button
            #        Of course, in his case no warning have to be emitted
            #warnings.warn('on_run: BEST parameters file not saved')
            return

        if not self.settings.bestbin:
            logging.error(
                'The BEST executable seems not to be correctly set: '
                'please fix it using the "Preferences" dialog.')
            return

        self.close_resultview()

        inifilename = self.inifilename
        self.run(inifilename)

    def on_stop(self, widget=None):
        self.stop()

    def on_set_default(self, widget=None):
        self.paramview.set_default()

    def on_clear(self, widget=None):
        # @TODO: fix for image preview
        self.paramview.clear()

    ### Callbacks for the Preferences menu ####################################
    def on_toolbar_style_changed(self, widget):
        for option in widget.get_group():
            if option.get_active():
                stylename = 'TOOLBAR_%s' % option.get_name()[13:].upper()
                style = getattr(gtk, stylename)
                self.toolbar.set_style(style)

    def on_show_outputplane(self, widget):
        # widget in (self.show_output_menuitem, self.show_output_button):
        if widget.get_active():
            fraction = self.settings.outputplane_fraction
            if not fraction:
                fraction = 0.2
        else:
            fraction = 0.
        size = self.vpaned.get_property('max-position')
        self.vpaned.set_position(int((1.-fraction) * size))

    def on_hide_outputplane(self, widget=None):
        self.show_outputplane(False)

    def on_show_toolbars(self, widget=None):
        if widget.get_active():
            self.handlebox.show()
        else:
            self.handlebox.hide()

    def on_set_preferences(self, widget=None):
        self.edit_options()

    ### Callbacks for the Help menu ###########################################
    def on_help(self, widget=None):
        url = utils.get_docfile('manual.html')
        self.open_url(url)

    def on_show_homepage(self, widget=None):
        url = 'http://bestgui.sourceforge.net'
        self.open_url(url)

    def on_about(self, widget=None):
        if self.settings.bestbin:
            best_version = utils.get_best_version(self.settings.bestbin,
                                                  self.settings.best_env)
        else:
            best_version = None
        dialog = dialogs.AboutDialog(best_version)
        dialog.run()
        dialog.destroy()

    ### Other callbacks #######################################################
    def on_delete_event(self, widget=None, event=None):
        # @TODO: check (button quit doesen't call this
        #print 'on_delete_event'
        try:
            if self.subprocess:
                msg = ('A task is still running.\n'
                       'Do you want to stop it and close the applicayion?')
                dialog = gtk.MessageDialog(
                        parent  = self.mainwin,
                        type    = gtk.MESSAGE_QUESTION,
                        flags   = gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                        buttons = gtk.BUTTONS_YES_NO,
                        message_format = msg)
                dialog.set_title('BESTGUI')
                response = dialog.run()
                dialog.destroy()
                if response == gtk.RESPONSE_NO:
                    return True
        except: # @TODO: check
            pass

        self.mainwin.unmaximize()
        self.save_window_state()

    def on_outputplane_resized(self, widget=None, data=None):
        if self.outputplane_isopen():
            self.show_output_menuitem.set_active(True)
            self.show_output_button.set_active(True)

            pos = self.vpaned.get_position()
            maxpos = self.vpaned.get_property('max-position')
            self.settings.outputplane_fraction = 1. - float(pos) / maxpos
        else:
            self.show_output_menuitem.set_active(False)
            self.show_output_button.set_active(False)

def main():
    from optparse import OptionParser
    # @TODO: see /usr/share/doc/python-gobject/examples/option.py
    # from gobject.option import OptionGroup, OptionParser, make_option

    parser = OptionParser(prog=info.name.lower(),
                          version='%s %s' % (info.name, info.version),
                          usage='%prog [options] [FILENAME]',
                          description=info.short_description)

    # NOTE: py2exe binary use this option
    # @TODO: fix
    parser.add_option('-u', action='store_true', help='silently ignored')
    parser.add_option('-d', '--debug', action='store_true', default=False, help='enable debug mode')
    parser.add_option('--logfile', help='print logs on file rather than on the stdout')

    (options, args) = parser.parse_args()
    # Setup logging
    if options.debug:
        level = logging.DEBUG
    else:
        level = logging.WARNING

    if not options.logfile and getattr(sys, 'frozen', False):
        import tempfile
        options.logfile = os.path.join(tempfile.gettempdir(),
                                       'tmp-bestgui-log.txt')

    # @WARNING: this seems to break the py2exe standalone binary
    # @TODO: check
    logging.basicConfig(level=level, format=BestGUI.LOGFORMAT,
                        datefmt=BestGUI.LOGDATEFMT, filename=options.logfile)
    logging.debug('bestgui start')

    try:
        # Define and set the except hook
        sysexcepthook = sys.excepthook
        def excepthook(etype, evalue, etraceback, sysexcepthook=sys.excepthook):
            try:
                try:
                    version = '%s.%s' % (info.version, __version__.split()[-2])
                    banner = ('# BESTGUI v. %s\n'
                              '# Unhandled ecception report generated on %s\n'
                              '\n') % (version, time.asctime())
                    dialogs.run_unhandledexception_dialog(
                                None, banner, etype, evalue, etraceback)
                    sysexcepthook(etype, evalue, etraceback)
                except:
                    sys.stderr.write('Exception in "excepthook".\n')
                    traceback.print_exc(file=sys.stderr)
            finally:
                gtk.main_quit()
                sys.exit(1)
        sys.excepthook = excepthook

        if sys.platform[:3] == 'win':
            # Redirect the stderr to the stdout
            # This is a workaround to avoid that the standalone binary
            # generated by py2exe pops up a message window when application
            # is closed.
            sys.stderr = sys.stdout

        # Force the english language
        if os.environ.get('LANGUAGE') != 'en':
            os.environ['LANGUAGE'] = 'en'
            os.environ['LANG'] = 'C'

            if sys.platform[:3] == 'win':
                # Workaround for quoting issues
                path, executable = os.path.split(sys.executable)
                if path not in os.getenv('PATH', '').split(os.pathsep):
                    os.environ['PATH'] = os.pathsep.join(
                                                [os.getenv('PATH', ''), path])

                if getattr(sys, 'frozen', False):
                    # handle paths with spaces (??)
                    args = ['"%s"' % sys.argv[0]] + sys.argv[1:]
                else:
                    args = [executable, '-u'] + \
                                            [u'"%s"' % arg for arg in sys.argv]
                logging.debug('call "execvpe" with args = %s' % args)
                os.execvpe(executable, args, os.environ)

        # Force numeric locale settings
        locale.setlocale(locale.LC_NUMERIC, 'C')

        # Instantiate the application
        app = BestGUI(level)
    except:
        gobject.timeout_add(100, excepthook, *sys.exc_info())
        gtk.main()
    else:
        sys.excepthook = sysexcepthook
        if len(args) > 0:
            app.open(args[0])
        app.main()
        logging.debug('bestgui close')
        sys.exit(0)

if(__name__ == '__main__'):
    main()
