### Copyright (C) 2006-2009 Antonio Valentino <a_valentino@users.sf.net>

### This file is part of BESTGUI.

### BESTGUI is free software; you can redistribute it and/or modify
### it under the terms of the GNU General Public License as published by
### the Free Software Foundation; either version 2 of the License, or
### (at your option) any later version.

### BESTGUI is distributed in the hope that it will be useful,
### but WITHOUT ANY WARRANTY; without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
### GNU General Public License for more details.

### You should have received a copy of the GNU General Public License
### along with BESTGUI; if not, write to the Free Software
### Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.

'''Dialogs classes and utilities.'''

__author__  = 'Antonio Valentino <a_valentino@users.sf.net>'
__date__    = '$Date: 2009-01-03 12:54:43 +0100 (sab, 03 gen 2009) $'
__version__ = '$Revision: 700 $'

import os
import sys
import traceback

import gtk

import info
import gladewrap
from utils import get_best_version, get_iconfile, get_docfile, get_gladefile

class AboutDialog(gtk.AboutDialog):
    def __init__(self, bestversion=None):
        gtk.AboutDialog.__init__(self)

        # Set the logo
        try:
            # The gtk+-2.10.13-setup.exe package from gimp-win.sf.net doesn't
            # provide the svg loader.
            # @TODO: fix
            if sys.platform[:3] == 'win':
                iconfilename = get_iconfile('bestguilogo128.png')
            else:
                iconfilename = get_iconfile('bestguilogo.svg')
            logo = gtk.gdk.pixbuf_new_from_file(iconfilename)
        except (IOError, OSError), e:
            run_error_dialog(e)
            logo = None

        # Set the BEST version
        executable_name = 'BEST'
        if not bestversion:
            bestversion = get_best_version()
        if not bestversion:
            bestversion = 'unknown'
        else:
            # workaround for the program name based on the program version
            if bestversion < '4.0':
                executable_name = 'STBX'

        # Set the message

        msglines = [
            info.short_description,
            '',
            '%s %s (http://earth.esa.int/services/best)' %
                                (executable_name, bestversion),
        ]
        msglines.extend(info.all_versions)
        msg = '\n'.join(msglines)

        self.set_authors(info.authors)
        self.set_comments(msg)
        self.set_copyright(info.copyright)
        #self.set_documenters(info.documenters)
        self.set_license(info.license)
        self.set_logo(logo)
        #self.set_logo_icon_name('icons/envisat.gif')
        self.set_name(info.name)
        #self.set_translator_credits(info.translator_credits)
        self.set_version(info.version)
        self.set_website(info.website)
        self.set_website_label(info.website_label)


def add_filechooser_filters(filechooser, patterns):
    for pattern, name in patterns:
        filefilter = gtk.FileFilter()
        filefilter.set_name(name)
        filefilter.add_pattern(pattern)
        filechooser.add_filter(filefilter)

def run_save_dialog(filechooser):
    # @COMPATIBILITY: gtk.FileChooser.set_do_overwrite_confirmation is
    #                 avalilable in PyGTK 2.8 and above
    if gtk.pygtk_version >= (2,8,0):
        filechooser.set_do_overwrite_confirmation(True)
        response = filechooser.run()
        if(response == gtk.RESPONSE_CANCEL):
            return None
        filename = filechooser.get_filename()
    else:
        filename = None
        while not filename:
            response = filechooser.run()
            if(response == gtk.RESPONSE_CANCEL):
                return None
            filename = filechooser.get_filename()
            if filename and os.path.exists(filename):
                if os.path.isdir(filename):
                    msg = ('Directory "%s" already exists.\n\n'
                           'Are you sure you want overwrite it?') % filename
                else:
                    msg = ('File "%s" already exists.\n\n'
                           'Are you sure you want overwrite it?') % filename
                msgdialog = gtk.MessageDialog(
                                    parent  = filechooser,
                                    flags   = gtk.DIALOG_MODAL |
                                              gtk.DIALOG_DESTROY_WITH_PARENT,
                                    type    = gtk.MESSAGE_QUESTION ,
                                    buttons = gtk.BUTTONS_YES_NO,
                                    message_format = msg)
                msgdialog.set_default_response(gtk.RESPONSE_NO)
                msgdialog_response = msgdialog.run()
                msgdialog.destroy()
                if(msgdialog_response != gtk.RESPONSE_YES):
                    filename = None
    filechooser.hide()
    return filename

class UnhandledExceptionDialog(gtk.Dialog):
    '''@TODO: see xpn, use glade, rename ExceptionDialog or ErrorDialog'''
    def __init__(self, title='Unhandled exception caught', parent=None,
                 etype=None, evalue=None, etraceback=None):
        super(UnhandledExceptionDialog, self).__init__(
                title,
                parent,
                flags   = gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                buttons = (gtk.STOCK_SAVE_AS, gtk.RESPONSE_OK,
                           gtk.STOCK_CLOSE,   gtk.RESPONSE_CLOSE))
        self.set_default_response(gtk.RESPONSE_CLOSE)
        self.set_default_size(500, 300)
        if etype is None and evalue is None and etraceback is None:
            etype, evalue, etraceback = sys.exc_info()

        textview    = gtk.TextView()
        scrolledwin = gtk.ScrolledWindow()
        scrolledwin.add_with_viewport(textview)
        scrolledwin.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolledwin.show_all()
        self.vbox.pack_start(scrolledwin)
        msg = ''.join(traceback.format_exception(etype, evalue, etraceback))
        self.buffer = textview.get_buffer()
        self.buffer.set_text(msg)

    def save_report(self, banner=None):
        text = self.buffer.get_text(self.buffer.get_start_iter(),
                                    self.buffer.get_end_iter())
        filechooser = gtk.FileChooserDialog(
                            title   = 'Save Ecxeption Report',
                            parent  = self,
                            action  = gtk.FILE_CHOOSER_ACTION_SAVE,
                            buttons = (gtk.STOCK_OK,     gtk.RESPONSE_OK,
                                       gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
        patterns = [('*.txt', 'Text files'), ('*', 'All Files')]
        add_filechooser_filters(filechooser, patterns)
        filechooser.set_current_name('bestgui-exception-report.txt')
        filechooser.set_select_multiple(False)
        filechooser.set_default_response(gtk.RESPONSE_OK)
        filename = run_save_dialog(filechooser)
        filechooser.destroy()
        if filename:
            fd = open(filename, 'w')
            if banner:
                fd.write(banner)
            fd.write(text)
            fd.close()

def run_unhandledexception_dialog(parent=None, banner=None,
                                  etype=None, evalue=None, etraceback=None):
    if(etype is None) and (evalue is None) and (etraceback is None):
        etype, evalue, etraceback = sys.exc_info()
    dialog = UnhandledExceptionDialog('Unhandled exception caught', parent,
                                      etype, evalue, etraceback)
    response = dialog.run()
    while(response != gtk.RESPONSE_CLOSE):
        dialog.save_report(banner)
        response = dialog.run()
    dialog.destroy()

def run_message_dialog(message, parent=None, dtype=gtk.MESSAGE_INFO,
                       title='BESTGUI'):
    if not parent:
        toplevels = gtk.window_list_toplevels()
        if toplevels:
            parent = toplevels[0]
    dialog = gtk.MessageDialog(
                    parent  = parent,
                    type    = dtype,
                    flags   = gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                    buttons = gtk.BUTTONS_CLOSE,
                    message_format = str(message))
    dialog.set_title(title)
    dialog.run()
    dialog.destroy()

def run_error_dialog(message=None, parent=None):
    if not message:
        message = ''.join(traceback.format_exception(*sys.exc_info()))
    run_message_dialog(message, parent, dtype=gtk.MESSAGE_ERROR)

class EditCoorDialog(gtk.Dialog):
    def __init__(self, title   = 'Enter coordinates',
                       parent  = None,
                       coorsys = 'ROWCOL',
                       coor    = None):
        super(EditCoorDialog, self).__init__(
                title,
                parent,
                flags   = gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                buttons = (gtk.STOCK_OK,     gtk.RESPONSE_OK,
                           gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
        self.set_default_response(gtk.RESPONSE_CANCEL)
        #self.set_default_size(500,300)
        gladefile = get_gladefile('bestgui.glade')
        self.coorform = gladewrap.GladeObject(gladefile, 'centre_hbox')
        self.coorform.centre_coor1_spinbutton.connect('activate',
                                            self.on_activate, gtk.RESPONSE_OK)
        self.coorform.centre_coor2_spinbutton.connect('activate',
                                            self.on_activate, gtk.RESPONSE_OK)
        self.coorform.widget.show_all()
        self.vbox.pack_start(self.coorform.widget)
        self.set_coorsys(coorsys)
        if coor:
            self.set_coor(*coor)

    def get_coorsys(self):
        label = self.coorform.centre_coor1_label.get_text()
        if label == 'Row':
            return('ROWCOL')
        if label == 'Latitude':
            return('LATLON')
        return None

    def set_coorsys(self, coorsys):
        form = self.coorform
        if coorsys == 'ROWCOL':
            form.centre_coor1_label.set_text('Row')
            form.centre_coor2_label.set_text('Column')
            form.centre_coor1_spinbutton.set_digits(0)
            form.centre_coor1_spinbutton.set_range(0, 100000)
            form.centre_coor2_spinbutton.set_digits(0)
            form.centre_coor2_spinbutton.set_range(0, 100000)
        elif coorsys == 'LATLON':
            form.centre_coor1_label.set_text('Latitude')
            form.centre_coor2_label.set_text('Longitude')
            form.centre_coor1_spinbutton.set_digits(4)
            form.centre_coor1_spinbutton.set_range(-90, 90)
            form.centre_coor2_spinbutton.set_digits(4)
            form.centre_coor2_spinbutton.set_range(-90, 90)
        else:
            raise ValueError(('Invalid coorsys "%s" ("ROWCOL" or "LATLON" '
                              'expected.') % coorsys)

    def get_coor(self):
        form = self.coorform
        coorsys = self.get_coorsys()
        if coorsys == 'ROWCOL':
            return(form.centre_coor1_spinbutton.get_value_as_int(),
                   form.centre_coor2_spinbutton.get_value_as_int())
        return(form.centre_coor1_spinbutton.get_value(),
               form.centre_coor2_spinbutton.get_value())

    def set_coor(self, coor1, coor2):
        form = self.coorform
        coorsys = self.get_coorsys()
        if coorsys == 'ROWCOL':
            form.centre_coor1_spinbutton.set_value(int(coor1))
            form.centre_coor2_spinbutton.set_value(int(coor2))
        elif coorsys == 'LATLON':
            form.centre_coor1_spinbutton.set_value(float(coor1))
            form.centre_coor2_spinbutton.set_value(float(coor2))
        else:
            raise ValueError(('Invalid coorsys "%s" ("ROWCOL" or "LATLON" '
                              'expected.') % coorsys)

    def on_activate(self, widget=None, data=None):
        self.response(data)

if(__name__ == '__main__'):
    def run_about_dialog():
        try:
            dialog = AboutDialog()
            dialog.run()
            dialog.destroy()
        finally:
            gtk.main_quit()
    import gobject
    gobject.timeout_add(0, run_about_dialog)
    gtk.main()
