### Copyright (C) 2006-2009 Antonio Valentino <a_valentino@users.sf.net>

### This file is part of BESTGUI.

### BESTGUI is free software; you can redistribute it and/or modify
### it under the terms of the GNU General Public License as published by
### the Free Software Foundation; either version 2 of the License, or
### (at your option) any later version.

### BESTGUI is distributed in the hope that it will be useful,
### but WITHOUT ANY WARRANTY; without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
### GNU General Public License for more details.

### You should have received a copy of the GNU General Public License
### along with BESTGUI; if not, write to the Free Software
### Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.

'''Module that provides an object oriented abstraction to pygtk and libglade.'''

__author__  = 'Antonio Valentino <a_valentino@users.sf.net>'
__version__ = '$Revision: 704 $'
__date__    = '$Date: 2009-04-13 10:09:18 +0200 (lun, 13 apr 2009) $'

import gtk
import gtk.glade

class GladeObject(object):
    '''Wrap a GTK based GUI built with glade into a python object.

    GladeObject allows you to access all the widgets in the GUI as attributes.
    Further all the callbacks defined in the sub-class of GladeObject are
    automatically connected at instantiation time.

    NOTE: GladeObject doesn't support multiple inheritance.

    '''

    def __init__(self, gladefile, root, domain='', typedict={}):
        #super(GladeObject, self).__init__()  # @TODO: check
        if isinstance(gladefile, gtk.glade.XML):
            self.xml = gladefile
        else:
            self.xml = gtk.glade.XML(gladefile, root, domain, typedict)
        self.xml.signal_autoconnect(self)
        self.widget = self.xml.get_widget(root)
        assert self.widget, 'Unable to find the root widget: %s.' % root

    def __getattr__(self, name):
        widget = self.xml.get_widget(name)
        if widget: # cache lookups
            setattr(self, name, widget)
            return widget
        raise AttributeError(name)

class GladeDialog(GladeObject):
    def __init__(self, gladefile, root, domain='', typedict={}):
        super(GladeDialog, self).__init__(gladefile, root, domain, typedict)

    def __getattr__(self, name):
        try:
            return super(GladeDialog, self).__getattr__(name)
        except AttributeError:
            return getattr(self.widget, name)

class GtkGladeApp(GladeObject):
    def __init__(self, gladefile, root, version=None, domain='', typedict={}):
        super(GtkGladeApp, self).__init__(gladefile, root, domain, typedict)
        self.widget.connect('destroy', self.quit)
        if hasattr(self, 'on_delete_event'):
            self.widget.connect('delete_event', self.on_delete_event)
        self.version = version

    def main(self):
        gtk.main()

    def quit(self, *args):
        gtk.main_quit()

    def flush_events(self):
        while gtk.events_pending():
            gtk.main_iteration()
