### Copyright (C) 2006-2009 Antonio Valentino <a_valentino@users.sf.net>

### This file is part of BESTGUI.

### BESTGUI is free software; you can redistribute it and/or modify
### it under the terms of the GNU General Public License as published by
### the Free Software Foundation; either version 2 of the License, or
### (at your option) any later version.

### BESTGUI is distributed in the hope that it will be useful,
### but WITHOUT ANY WARRANTY; without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
### GNU General Public License for more details.

### You should have received a copy of the GNU General Public License
### along with BESTGUI; if not, write to the Free Software
### Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.

'''Info about the BASTGUI program.'''

__author__  = 'Antonio Valentino <a_valentino@users.sf.net>'
__date__    = '$Date: 2009-04-13 11:51:36 +0200 (lun, 13 apr 2009) $'
__version__ = '$Revision: 706 $'

import sys
import gtk

def _get_authors():
    from utils import get_docfile  # @TODO: fix
    try:
        authors_ = open(get_docfile('AUTHORS')).read()
        authors_ = authors_.splitlines()
    except (IOError, OSError), e:
        import logging
        logging.error(e)
        authors_ = (__author__,)
    return authors_

def _get_license():
    import os
    import gzip
    from utils import get_docfile # @TODO: fix
    try:
        licensefile = get_docfile('LICENSE.txt')
        if not os.path.exists(licensefile):
            # When generate the debian package using 'alien' the
            # README.txt file is gzipped
            license_ = gzip.open(licensefile + '.gz').read()
        else:
            license_ = open(licensefile).read()
    except (IOError, OSError), e:
        import logging
        logging.error(e)
        license_ = None
    return license_

#artists = None
authors = _get_authors()
author_name = [name.split(' <')[0] for name in authors]
author_email = [name.split(' <')[-1].rstrip('>') for name in authors]
copyright = ('Copytight (C) 2006-2009 %s' %
                ', '.join([name.split('<')[0].rstrip() for name in authors]))
short_description = 'A GUI front-end for BEST written in Python and GTK+.'
description = '''BESTGUI is a GUI front-end for BEST written in Python and GTK+.

BEST (Basic Envisat SAR Toolbox) is a collection of executable software
tools that has been designed to facilitate the use of ESA (the European
Space Agency) SAR data. It operates according to user-generated
parameter files.

BESTGUI allows you to easily generate the parameters files for the BEST
tools and launch the processing from the GUI. It provides

- enhanced menus and toolbars
- persistent preferences
- smart environment handling
- colorized output log
- progress bars for processing completion
- the ability of stopping a running task from the GUI (stop button)
- a dialog for the AoI specification
- output preview for all (and not only) the tools that have an image as
  output

'''

#documenters = ('AV',)
license = _get_license()
license_type = 'GNU GPL'
name = 'BESTGUI'
#thanks = None
translator_credits = None
version = '1.2.1'
website = 'http://bestgui.sourceforge.net'
website_label = 'http://bestgui.sourceforge.net'

all_versions = (
    #'%s v. %s (http://earth.esa.int/services/best)' % (binName, bestVersion),
    'Python v. %s (www.python.org)' % '.'.join(map(str,sys.version_info[:3])),
    'PyGTK v. %s (www.pygtk.org)' % '.'.join(map(str, gtk.pygtk_version)),
    'GTK v. %s (www.gtk.org)' % '.'.join(map(str, gtk.gtk_version)),
)

del sys, gtk
