### Copyright (C) 2006-2009 Antonio Valentino <a_valentino@users.sf.net>

### This file is part of BESTGUI.

### BESTGUI is free software; you can redistribute it and/or modify
### it under the terms of the GNU General Public License as published by
### the Free Software Foundation; either version 2 of the License, or
### (at your option) any later version.

### BESTGUI is distributed in the hope that it will be useful,
### but WITHOUT ANY WARRANTY; without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
### GNU General Public License for more details.

### You should have received a copy of the GNU General Public License
### along with BESTGUI; if not, write to the Free Software
### Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.

'''Menu definition and utiliti functions.'''

__author__  = 'Antonio Valentino <a_valentino@users.sf.net>'
__date__    = '$Date: 2009-04-13 10:09:18 +0200 (lun, 13 apr 2009) $'
__version__ = '$Revision: 704 $'

import ConfigParser
from operator import attrgetter

import gtk

import utils

# @TODO: __ALL__

def add_menu(parent, name):
    # @TODO: rewrite
    menu     = gtk.Menu()
    menu.set_title(name)
    menuitem = gtk.MenuItem(name)
    menuitem.set_submenu(menu)
    menuitem.show()
    parent.append(menuitem)
    return(menu)

def add_menu_item(parent, name, command, data):
    menuitem = gtk.MenuItem(name)
    menuitem.connect('activate', command, data)
    menuitem.show()
    parent.append(menuitem)
    return(menuitem)

class MenuDescriptor(object):
    def __init__(self, name, parent, label='', position=None):
        super(MenuDescriptor, self).__init__()
        self.name     = name
        self.parent   = parent
        self.label    = label
        self.position = position
        self.children = []

    def __repr__(self):
        attrs = []
        for attr in ('name', 'parent', 'label', 'position'):
            attr = getattr(self, attr)
            if attr:
                attrs.append("'%s'," % attr)
            else:
                attrs.append(str(attr))
        attrs.insert(0, self.__class__.__name__)
        toolstr = "%s(%-15s%-15s%-20s%-5s)" % tuple(attrs)
        if self.children:
            childrenstr = '\n'.join([repr(child) for child in self.children])
            toolstr = '%s\n%s' % (toolstr, childrenstr)
        return toolstr

    def __str__(self):
        toolstr = self.label.replace('_','')
        if self.children:
            childrenstr = '\n'.join([str(child) for child in self.children])
            childrenstr = '\n'.join(['    %s' % line
                                        for line in childrenstr.splitlines()])
            toolstr = '%s\n%s' % (toolstr, childrenstr)
        return toolstr

class MenuItemDescriptor(object):
    def __init__(self, name, parent, label='', position=None, data=None):
        super(MenuItemDescriptor, self).__init__()
        self.name     = name
        self.parent   = parent
        self.label    = label
        self.position = position
        self.data     = data

    def __str__(self):
        return self.label.replace('_', '')

    def __repr__(self):
        attrs = []
        for attr in ('name', 'parent', 'label', 'position', 'data'):
            attr = getattr(self, attr)
            if attr:
                attrs.append("'%s'," % attr)
            else:
                attrs.append(str(attr))
        attrs.insert(0, self.__class__.__name__)
        return "%s(%-15s%-15s%-20s%-5s%s)" % tuple(attrs)

def get_menu_map():
    cfgfilename = utils.get_cfgfile('toolsmenu.ini')
    cfg = ConfigParser.SafeConfigParser()
    cfg.read(cfgfilename)

    menumap = dict(tools=MenuDescriptor('tools', None, '_Tools'))
    childrenmap = {}
    for section in cfg.sections():
        name     = cfg.get(section, 'name')
        parent   = cfg.get(section, 'parent')
        label    = cfg.get(section, 'label')
        position = cfg.get(section, 'position')
        position = int(position)
        newmenu = MenuDescriptor(name, parent, label, position)
        childrenmap.setdefault(parent, []).append(newmenu)
        menumap[name] = newmenu

    for children in childrenmap.values():
        children.sort(key=attrgetter('position'))

    for toolname in menumap:
        menumap[toolname].children = childrenmap.get(toolname, [])
    return menumap

def build_tools_menu(menumap, callback):
    toolsmenu = gtk.Menu()
    toolsmenu.set_title('_Tools')

    parentsmap = {'tools' : toolsmenu}

    def build_menu_entry(entry, command):
        parent = parentsmap[entry.parent]
        if hasattr(entry, 'children'):
            parentsmap[entry.name] = add_menu(parent, entry.label)
            for item in entry.children:
                build_menu_entry(item, command)
            return parentsmap[entry.name]
        else:
            return add_menu_item(parent, entry.label, command, entry.data)

    for child in menumap['tools'].children:
        build_menu_entry(child, callback)

    return toolsmenu
